
/*********************************************************************
 *
 *               Data I2C EEPROM Access Routines
 *
 *********************************************************************
 * FileName:        I2CEEPROM.c
 * Dependencies:    Compiler.h
 *                  XEEPROM.h
 * Processor:       PIC18
 * Complier:        Microchip C18 v3.03 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright  2002-2007 Microchip Technology Inc.  All rights 
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and 
 * distribute: 
 * (i)  the Software when embedded on a Microchip microcontroller or 
 *      digital signal controller product (DeviceE which is 
 *      integrated into Licensees product; or
 * (ii) ONLY the Software driver source files ENC28J60.c and 
 *      ENC28J60.h ported to a non-Microchip device used in 
 *      conjunction with a Microchip ethernet controller for the 
 *      sole purpose of interfacing with the ethernet controller. 
 *
 * You should refer to the license agreement accompanying this 
 * Software for additional information regarding your rights and 
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED AS ISEWITHOUT 
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT 
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL 
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR 
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF 
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS 
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE 
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER 
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT 
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Nilesh Rajbharti     5/20/02     Original (Rev. 1.0)
 * Howard Schlunder		8/10/06		Renamed registers/bits to use 
 *									C18/C30 style standard names
********************************************************************/
#include "Include/TCPIP Stack/TCPIP.h"

#if !defined(OLD_I2C_Routines)
// ****Vٰ**** i[`774sڈȍ~j

#if defined(MPFS_USE_EEPROM) && defined(EEPROM_SCL_TRIS) && defined(STACK_USE_MPFS2)		//MPFS2

	// 24LC256 I/O pins
	#define EEPROM_SCL_TRIS		(TRISCbits.TRISC3)
	#define EEPROM_SDA_TRIS		(TRISCbits.TRISC4)
	#define EEPROM_SCL_IO		(PORTCbits.RC3)
	#define EEPROM_SDA_IO		(PORTCbits.RC4)
	#define EEPROM_SSPBUF		(SSPBUF)
	#define EEPROM_SSPCON1		(SSPCON1)
	#define EEPROM_SSPCON1bits	(SSPCON1bits)
	#define EEPROM_SSPCON2		(SSPCON2)
	#define EEPROM_SSPCON2bits	(SSPCON2bits)
	#define EEPROM_SSPSTATbits	(SSPSTATbits)


// I2C bits are in different registers for 8-bit and 16-bit 
// microcontrollers
#if defined(__18CXX)
	#define I2CBIT_ACKDT		(EEPROM_SSPCON2bits.ACKDT)
	#define I2CBIT_ACKEN		(EEPROM_SSPCON2bits.ACKEN)
	#define I2CBIT_RCEN			(EEPROM_SSPCON2bits.RCEN)
	#define I2CBIT_PEN			(EEPROM_SSPCON2bits.PEN)
	#define I2CBIT_SEN			(EEPROM_SSPCON2bits.SEN)
	#define I2CBIT_ACKSTAT		(EEPROM_SSPCON2bits.ACKSTAT)
	#define I2CBIT_TRSTAT		(EEPROM_SSPSTATbits.R_W)
//	#define I2CBIT_TRSTAT		(EEPROM_SSPSTATbits.BF)
	#define I2CBIT_IWCOL		(EEPROM_SSPCON1bits.WCOL)
	#define I2C_DATA_IN			(EEPROM_SSPBUF)
	#define I2C_DATA_OUT		(EEPROM_SSPBUF)
#elif defined(__C30__)
	#define I2CBIT_ACKDT		(EEPROM_I2CCONbits.ACKDT)
	#define I2CBIT_ACKEN		(EEPROM_I2CCONbits.ACKEN)
	#define I2CBIT_RCEN			(EEPROM_I2CCONbits.RCEN)
	#define I2CBIT_PEN			(EEPROM_I2CCONbits.PEN)
	#define I2CBIT_SEN			(EEPROM_I2CCONbits.SEN)
	#define I2CBIT_ACKSTAT		(EEPROM_I2CSTATbits.ACKSTAT)
	#define I2CBIT_TRSTAT		(EEPROM_I2CSTATbits.TRSTAT)
	#define I2CBIT_IWCOL		(EEPROM_I2CSTATbits.IWCOL)
	#define I2C_DATA_IN			(EEPROM_I2CRCV)
	#define I2C_DATA_OUT		(EEPROM_I2CTRN)
#endif


#define READ	(0x01)
#define WRITE	(0x00)

static BOOL NeedACK = FALSE;
static BOOL NeedEndRead = FALSE;
static BOOL NeedEndWrite = FALSE;

static void Delayus(BYTE v);
static void I2CStart(void);
static void I2CStop(void);
static void I2CAck(BOOL Not);
static BOOL I2CPut(BYTE c);
static BYTE I2CGet(void);

static void Delayus(BYTE v)
{
	while(v--)
	{
		Nop();
	}
}


static void I2CStart(void)
{
	EEPROM_SCL_TRIS = 1;	// Clock line must be high
	while(EEPROM_SCL_IO == 0);	
	Delayus(3);

	// Generate a start condition
	EEPROM_SDA_TRIS = 0;
	Delayus(3);	
}

static void I2CStop(void)
{
	// Pull data low
	EEPROM_SDA_TRIS = 0;
	Delayus(1);


	EEPROM_SCL_TRIS = 1;	// Clock line must be high
	while(EEPROM_SCL_IO == 0);	
	Delayus(3);

	// Generate a stop condition
	EEPROM_SDA_TRIS = 1;
	Delayus(3);	
}

static void I2CAck(BOOL Not)
{
	// Generate the (N)Ack
	EEPROM_SDA_TRIS = Not;
	Delayus(3);	

	EEPROM_SCL_TRIS = 1;	// Release the clock
	while(EEPROM_SCL_IO == 0);	
	Delayus(3);	

	EEPROM_SCL_TRIS = 0;	// Pull clock low
	Delayus(1);	
	EEPROM_SDA_TRIS = 1;
	Delayus(3);	
}

static BOOL I2CPut(BYTE c)
{
	BYTE i;

	EEPROM_SCL_TRIS = 0;	// Clock line must be low
	Delayus(1);

	for(i = 0; i < 8; i++)
	{	
		// Generate the data bit
		EEPROM_SDA_TRIS = ((BYTE_VAL*)&c)->bits.b7;
		Delayus(3);
		EEPROM_SCL_TRIS = 1;	// Release the clock	
		while(EEPROM_SCL_IO == 0);	
		Delayus(3);
		EEPROM_SCL_TRIS = 0;	// Pull clock low
		Delayus(1);
		c <<= 1;
	}
	
	// Read the ACK response
	EEPROM_SDA_TRIS = 1;
	Delayus(3);
	
	EEPROM_SCL_TRIS = 1;	// Release the clock	
	while(EEPROM_SCL_IO == 0);	
	Delayus(1);
	i = EEPROM_SDA_IO;
	Delayus(1);
	i += EEPROM_SDA_IO;
	Delayus(1);
	i += EEPROM_SDA_IO;
	
	EEPROM_SCL_TRIS = 0;	// Pull clock low
	Delayus(3);

	return i >= 2;
}


static BYTE I2CGet(void)
{
	BYTE i;
	BYTE c;

	EEPROM_SCL_TRIS = 0;	// Clock line must be low
	Delayus(1);
	EEPROM_SDA_TRIS = 1;
	Delayus(3);

	for(i = 0; i < 8; i++)
	{
		EEPROM_SCL_TRIS = 1;	// Release the clock
		while(EEPROM_SCL_IO == 0);	
		Delayus(3);
	
		// Get the data bit
		c <<= 1;
		((BYTE_VAL*)&c)->bits.b0 = EEPROM_SDA_IO;
		EEPROM_SCL_TRIS = 0;	// Pull clock low
		Delayus(3);
	}

	return c;
}

void XEEInit(void)
{
	LATCbits.LATC3 = 0;
	LATCbits.LATC4 = 0;
//	EEPROM_SCL_IO = 0;
//	EEPROM_SDA_IO = 0;
	EEPROM_SCL_TRIS = 1;		// Set SCL pin to output
	EEPROM_SDA_TRIS = 1;		// Set SDA pin to input
}

XEE_RESULT XEEBeginWrite(XEE_ADDR address)
{
	if(NeedEndRead)
		while(1);
	if(NeedEndWrite)
		while(1);

	while(XEEIsBusy());

	NeedEndWrite = TRUE;

	// Generate a start condition
	I2CStart();

	// Send "control" byte with device address to slave
	if(I2CPut(EEPROM_CONTROL | WRITE))
		while(1);
	// Send address bytes
	if(I2CPut(((WORD_VAL*)&address)->v[1]))
		while(1);
	if(I2CPut(((WORD_VAL*)&address)->v[0]))
		while(1);
	
	return XEE_SUCCESS;
}

XEE_RESULT XEEBeginRead(XEE_ADDR address)
{
	// Set the address to read from.  This procedure is identical 
	// for reading and writing.
	XEEBeginWrite(address);

	NeedEndRead = TRUE;
	NeedEndWrite = FALSE;

	// Generate a start condition
	I2CStart();

	// Send "control" byte with device address to slave
	if(I2CPut(EEPROM_CONTROL | READ))
		while(1);
	
	return XEE_SUCCESS;
}

XEE_RESULT XEEEndWrite(void)
{
	if(!NeedEndWrite)
		while(1);
	if(NeedEndRead)
		while(1);

	NeedEndWrite = FALSE;

	// Generate a STOP condition
	I2CStop();
}

XEE_RESULT XEEWrite(unsigned char val)
{
	if(!NeedEndWrite)
		while(1);
	if(NeedEndRead)
		while(1);

	// Send data byte
	if(I2CPut(val))
		while(1);

	return XEE_SUCCESS;
}

unsigned char XEERead(void)
{
	if(NeedEndWrite)
		while(1);
	if(!NeedEndRead)
		while(1);

	if(NeedACK)
	{
		NeedACK = FALSE;
		I2CAck(FALSE);
	}

	NeedACK = TRUE;

	// Receive a byte	
	return I2CGet();
}

XEE_RESULT XEEEndRead(void)
{
	if(NeedEndWrite)
		while(1);
	if(!NeedEndRead)
		while(1);

	NeedEndRead = FALSE;

	// Send a NACK to complete the read
	I2CAck(TRUE);
	NeedACK = FALSE;

	// Generate a STOP condition
	I2CStop();

	DelayMs(1);//TODO: debug

    return XEE_SUCCESS;
}

XEE_RESULT XEEReadArray(XEE_ADDR address, unsigned char *buffer, unsigned char length)
{
    XEEBeginRead(address);
    while(length--)
        *buffer++ = XEERead();
    XEEEndRead();

    return XEE_SUCCESS;
}

BOOL XEEIsBusy(void)
{
	BOOL i;

	if(NeedEndWrite)
		while(1);
	if(NeedEndRead)
		while(1);

	// Generate a start condition
	I2CStart();

	// Send "control" byte with device address to slave
	i = I2CPut(EEPROM_CONTROL | WRITE);

	// Generate a STOP condition
	I2CStop();

	return i;
}

//
///*********************************************************************
// * Function:        void XEEInit(unsigned char baud)
// *
// * PreCondition:    None
// *
// * Input:           baud    - SSPADD value for bit rate.
// *
// * Output:          None
// *
// * Side Effects:    None
// *
// * Overview:        Initialize I2C module to communicate to serial
// *                  EEPROM.
// *
// * Note:            None
// ********************************************************************/
//void XEEInit(BYTE baud)
//{
//
//#if defined(__18CXX)
//	EEPROM_SCL_TRIS = 1;		// Set SCL pin to output
//	EEPROM_SDA_TRIS = 1;		// Set SDA pin to input
//	SSPCON1 = 0x28;				// Set SSPEN and set I2C Master mode
//	SSPADD = (INSTR_FREQ+I2CBAUD/2)/I2CBAUD - 1;
//#elif defined(__C30__)
//	EEPROM_I2CCONbits.I2CEN = 1;
//	#if defined(__dsPIC30F__)	
//		I2CBRG = ((INSTR_FREQ+I2CBAUD/2)/I2CBAUD - (INSTR_FREQ+1111111/2)/1111111)-1; //dsPIC30F only
//	#else						
//		I2CBRG = (INSTR_FREQ+I2CBAUD)/(2*I2CBAUD)-1	// PIC24, dsPIC33
//	#endif
//#endif
//}
//
//
///*********************************************************************
// * Macro:           XEE_RESULT XEEBeginWrite(XEE_ADDR address)
// *
// * PreCondition:    XEEInit() is already called.
// *
// * Input:           control     - data EEPROM control code
// *                  address     - address to where to write
// *
// * Output:          XEE_SUCCESS if successful
// *                  other value if failed.
// *
// * Side Effects:    None
// *
// * Overview:        Sets up internal address counter of EEPROM.
// *
// * Note:            This function does not release the I2C bus.
// *                  User must close XEEEndWrite() after this function
// *                  is called to relase the I2C bus.
// ********************************************************************/
//XEE_RESULT XEEBeginWrite(BYTE control, XEE_ADDR address)
//{
//	if(NeedEndRead)
//		while(1);
//	if(NeedEndWrite)
//		while(1);
//
//	NeedEndWrite = TRUE;
//
//	// Generate a start condition
//	I2CBIT_SEN = 1;
//    while(I2CBIT_SEN);
//
//	// Send "control" byte with device address to slave
//	do
//	{
//		I2CBIT_IWCOL = 0;
//		I2C_DATA_OUT = EEPROM_CONTROL | WRITE;
//	} while(I2CBIT_IWCOL);
//	while(I2CBIT_TRSTAT);
//	if(I2CBIT_ACKSTAT)
//		while(1);
//
//	// Send address bytes
//	do
//	{
//		I2CBIT_IWCOL = 0;
//		I2C_DATA_OUT = ((WORD_VAL*)&address)->v[1];
//	} while(I2CBIT_IWCOL);
//	while(I2CBIT_TRSTAT);
//	if(I2CBIT_ACKSTAT)
//		while(1);
//	do
//	{
//		I2CBIT_IWCOL = 0;
//		I2C_DATA_OUT = ((WORD_VAL*)&address)->v[0];
//	} while(I2CBIT_IWCOL);
//	while(I2CBIT_TRSTAT);
//	if(I2CBIT_ACKSTAT)
//		while(1);
//	
//	return XEE_SUCCESS;
//}
//
///*********************************************************************
// * Function:        XEE_RESULT XEEBeginRead(XEE_ADDR address)
// *
// * PreCondition:    XEEInit() is already called.
// *
// * Input:           control - EEPROM control and address code.
// *                  address - Address at which read is to be performed.
// *
// * Output:          XEE_SUCCESS if successful
// *                  other value if failed.
// *
// * Side Effects:    None
// *
// * Overview:        Sets internal address counter to given address.
// *                  Puts EEPROM in sequential read mode.
// *
// * Note:            This function does not release I2C bus.
// *                  User must call XEEEndRead() when read is not longer
// *                  needed; I2C bus will released after XEEEndRead()
// *                  is called.
// ********************************************************************/
//XEE_RESULT XEEBeginRead(BYTE control, XEE_ADDR address)
//{
//	if(NeedEndWrite)
//		while(1);
//	if(NeedEndRead)
//		while(1);
//
//
//	// Set the address to read from.  This procedure is identical 
//	// for reading and writing.
//	XEEBeginWrite(0, address);
//
//	NeedEndRead = TRUE;
//	NeedEndWrite = FALSE;
//
////	// Generate a STOP condition
////	I2CBIT_PEN = 1;
////	while(I2CBIT_PEN);
//
//
//	EEPROM_SCL_TRIS = 1;
//
//	// Generate a start condition
//	I2CBIT_SEN = 1;
//    while(I2CBIT_SEN);
//
//	// Send "control" byte with device address to slave
//	do
//	{
//		I2CBIT_IWCOL = 0;
//		I2C_DATA_OUT = EEPROM_CONTROL | READ;
//	} while(I2CBIT_IWCOL);
//	while(I2CBIT_TRSTAT);
//	if(I2CBIT_ACKSTAT)
//		while(1);
//	
//	return XEE_SUCCESS;
//}
//
//
//XEE_RESULT XEEWrite(unsigned char val)
//{
//	if(!NeedEndWrite)
//		while(1);
//	if(NeedEndRead)
//		while(1);
//
//	// Send data byte
//	do
//	{
//		I2CBIT_IWCOL = 0;
//		I2C_DATA_OUT = val;
//	} while(I2CBIT_IWCOL);
//	while(I2CBIT_TRSTAT);
//	if(I2CBIT_ACKSTAT)
//		while(1);
//
//	return XEE_SUCCESS;
//}
//
///*********************************************************************
// * Function:        XEE_RESULT XEEEndWrite(void)
// *
// * PreCondition:    XEEInit() && XEEBeginWrite() are already called.
// *
// * Input:           None
// *
// * Output:          XEE_SUCCESS if successful
// *                  other value if failed.
// *
// * Side Effects:    None
// *
// * Overview:        Instructs EEPROM to begin write cycle.
// *
// * Note:            Call this function after either page full of bytes
// *                  written or no more bytes are left to load.
// *                  This function initiates the write cycle.
// *                  User must call for XEEWait() to ensure that write
// *                  cycle is finished before calling any other
// *                  routine.
// ********************************************************************/
//XEE_RESULT XEEEndWrite(void)
//{
//	if(!NeedEndWrite)
//		while(1);
//	if(NeedEndRead)
//		while(1);
//
//	NeedEndWrite = FALSE;
//
//	// Generate a STOP condition
//	I2CBIT_PEN = 1;
//	while(I2CBIT_PEN);
//
//	DelayMs(5);
//
//}
//
//
//
///*********************************************************************
// * Function:        XEE_RESULT XEERead(void)
// *
// * PreCondition:    XEEInit() && XEEBeginRead() are already called.
// *
// * Input:           None
// *
// * Output:          XEE_SUCCESS if successful
// *                  other value if failed.
// *
// * Side Effects:    None
// *
// * Overview:        Reads next byte from EEPROM; internal address
// *                  is incremented by one.
// *
// * Note:            This function does not release I2C bus.
// *                  User must call XEEEndRead() when read is not longer
// *                  needed; I2C bus will released after XEEEndRead()
// *                  is called.
// ********************************************************************/
//unsigned char XEERead(void)
//{
//	volatile BYTE i;
//	
//	if(NeedEndWrite)
//		while(1);
//	if(!NeedEndRead)
//		while(1);
//
//	if(NeedACK)
//	{
//		NeedACK = FALSE;
//		I2CBIT_ACKDT = 0;
//		I2CBIT_ACKEN = 1;
//		while(I2CBIT_ACKEN);
//	}
//
//	// Receive a byte
//	I2CBIT_RCEN = 1;
//	while(I2CBIT_RCEN);
//	NeedACK = TRUE;
//
//	i = I2C_DATA_IN;
//    return i;
//}
//
///*********************************************************************
// * Function:        XEE_RESULT XEEEndRead(void)
// *
// * PreCondition:    XEEInit() && XEEBeginRead() are already called.
// *
// * Input:           None
// *
// * Output:          XEE_SUCCESS if successful
// *                  other value if failed.
// *
// * Side Effects:    None
// *
// * Overview:        Ends sequential read cycle.
// *
// * Note:            This function ends seuential cycle that was in
// *                  progress.  It releases I2C bus.
// ********************************************************************/
//XEE_RESULT XEEEndRead(void)
//{
//	if(NeedEndWrite)
//		while(1);
//	if(!NeedEndRead)
//		while(1);
//
//	NeedEndRead = FALSE;
//
//	// Send a NACK to complete the read
//	I2CBIT_ACKDT = 1;
//	I2CBIT_ACKEN = 1;
//	while(I2CBIT_ACKEN);
//	NeedACK = FALSE;
//
//	// Generate a STOP condition
//	I2CBIT_PEN = 1;
//	while(I2CBIT_PEN);
//
//    return XEE_SUCCESS;
//}
//
//
///*********************************************************************
// * Function:        XEE_RESULT XEEReadArray(XEE_ADDR address,
// *                                          unsigned char *buffer,
// *                                          unsigned char length)
// *
// * PreCondition:    XEEInit() is already called.
// *
// * Input:           control     - EEPROM control and address code.
// *                  address     - Address from where array is to be read
// *                  buffer      - Caller supplied buffer to hold the data
// *                  length      - Number of bytes to read.
// *
// * Output:          XEE_SUCCESS if successful
// *                  other value if failed.
// *
// * Side Effects:    None
// *
// * Overview:        Reads desired number of bytes in sequential mode.
// *                  This function performs all necessary steps
// *                  and releases the bus when finished.
// *
// * Note:            None
// ********************************************************************/
//XEE_RESULT XEEReadArray(BYTE control, XEE_ADDR address,
//                        unsigned char *buffer,
//                        unsigned char length)
//{
//    XEEBeginRead(0, address);
//    while(length--)
//        *buffer++ = XEERead();
//    XEEEndRead();
//
//    return XEE_SUCCESS;
//}
//
//
///*********************************************************************
// * Function:        BOOL XEEIsBusy(void)
// *
// * PreCondition:    XEEInit() is already called.
// *
// * Input:           None
// *
// * Output:          FALSE if EEPROM is not busy
// *                  TRUE if EEPROM is busy or operation failed
// *
// * Side Effects:    None
// *
// * Overview:        Requests ACK from EEPROM.
// *
// * Note:            None
// ********************************************************************/
//BOOL XEEIsBusy(BYTE control)
//{
//	if(NeedEndWrite)
//		while(1);
//	if(NeedEndRead)
//		while(1);
//
//	// Generate a start condition
//	I2CBIT_SEN = 1;
//    while(I2CBIT_SEN);
//
//	// Send "control" byte with device address to slave
//	do
//	{
//		I2CBIT_IWCOL = 0;
//		I2C_DATA_OUT = EEPROM_CONTROL | WRITE;
//	} while(I2CBIT_IWCOL);
//	while(I2CBIT_TRSTAT);
//
//	// Generate a STOP condition
//	I2CBIT_PEN = 1;
//	while(I2CBIT_PEN);
//
//	return I2CBIT_ACKSTAT;
//}
//

#endif //#if defined(MPFS_USE_EEPROM) && defined(EEPROM_SCL_TRIS) && defined(STACK_USE_MPFS)

#else

//********* ٰ *******************************

#undef XEE_SUCCESS
//I2C[`𗘗p鑤f̂XEE_SUCCESSȊÔ߁AV
//݊lAXEE_SUCCESSȊǑʂ߂lɂB
typedef enum _XEE_RESULT
{
    XEE_SUCCESS = 0,
    XEE_READY = 0,
    XEE_BUS_COLLISION,
    XEE_NAK,
    XEE_VERIFY_ERR,
    XEE_BUSY
} XEE_RESULT;

#define READ	(0x01)
#define WRITE	(0x00)

/*********************************************************************
 *
 *               Data I2C EEPROM Access Routines
 *
 *********************************************************************
 * FileName:        I2CEEPROM.c
 * Dependencies:    Compiler.h
 *                  XEEPROM.h
 * Processor:       PIC18
 * Complier:        Microchip C18 v3.03 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * This software is owned by Microchip Technology Inc. ("Microchip") 
 * and is supplied to you for use exclusively as described in the 
 * associated software agreement.  This software is protected by 
 * software and other intellectual property laws.  Any use in 
 * violation of the software license may subject the user to criminal 
 * sanctions as well as civil liability.  Copyright 2006 Microchip
 * Technology Inc.  All rights reserved.
 *
 * This software is provided "AS IS."  MICROCHIP DISCLAIMS ALL 
 * WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, NOT LIMITED 
 * TO MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND 
 * INFRINGEMENT.  Microchip shall in no event be liable for special, 
 * incidental, or consequential damages.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Nilesh Rajbharti     5/20/02     Original (Rev. 1.0)
 * Howard Schlunder		8/10/06		Renamed registers/bits to use 
 *									C18/C30 style standard names
********************************************************************/
#include "Include/TCPIP Stack/TCPIP.h"

#if defined(MPFS_USE_EEPROM) || defined(MACADD_USE_EEPROM)		//1925

/*
#if !defined(PICDEMNET)
	#error "PICDEMNET is not defined, but I2CEEPROM.c is included.  Was SPIEEPROM.c supposed to be included instead?"
#endif
*/

#define IdleI2C()       while( (EEPROM_SPICON2 & 0x1F) | (EEPROM_SPISTATbits.R_W) );
#define StartI2C()      EEPROM_SPICON2bits.SEN=1
#define RestartI2C()    EEPROM_SPICON2bits.RSEN=1
#define StopI2C()       EEPROM_SPICON2bits.PEN=1
#define NotAckI2C()     EEPROM_SPICON2bits.ACKDT=1, EEPROM_SPICON2bits.ACKEN=1

static unsigned char getcI2C( void );
static unsigned char WriteI2C( unsigned char data_out );
XEE_RESULT XEEClose(void);



/*********************************************************************
 * Function:        void XEEInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Initialize I2C module to communicate to serial
 *                  EEPROM.
 *
 * Note:            None
 ********************************************************************/
#define   MASTER    8     /* I2C Master mode                    */
/* SSPSTAT REGISTER */
#define   SLEW_OFF  0xC0  /* Slew rate disabled for 100kHz mode */
#define   SLEW_ON   0x00  /* Slew rate enabled for 400kHz mode  */
#define   SSPENB    0x20  /* Enable serial port and configures
                             SCK, SDO, SDI                      */
void XEEInit(void)
{
  SSPSTAT &= 0x3F;                // power on state
  SSPCON1 = 0x00;                 // power on state
  EEPROM_SPICON2 = 0x00;                 // power on state
  SSPCON1 |= MASTER;            // select serial mode
  SSPSTAT |= SLEW_ON;                // slew rate on/off

  EEPROM_SCL_TRIS = 1;           // Set SCL (PORTC,3) pin to input
  EEPROM_SDA_TRIS = 1;           // Set SDA (PORTC,4) pin to input

  SSPCON1 |= SSPENB;              // enable synchronous serial port

  SSPADD = ((CLOCK_FREQ/400000)/4)-1;
}


/*********************************************************************
 * Function:        XEE_RESULT XEESetAddr(unsigned char control,
 *                                        XEE_ADDR address)
 *
 * PreCondition:    XEEInit() is already called.
 *
 * Input:           control     - data EEPROM control code
 *                  address     - address to be set
 *
 * Output:          XEE_SUCCESS if successful
 *                  other value if failed.
 *
 * Side Effects:    None
 *
 * Overview:        Modifies internal address counter of EEPROM.
 *
 * Note:            This function does not release the I2C bus.
 *                  User must close XEEClose() after this function
 *                  is called.
 ********************************************************************/
XEE_RESULT XEESetAddr(XEE_ADDR address)
{
    union
    {
        unsigned short int Val;
        struct
        {
            unsigned char LSB;
            unsigned char MSB;
        } bytes;
    } tempAddress;

    tempAddress.Val = address;

    IdleI2C();                      // ensure module is idle
    StartI2C();                     // initiate START condition
    while ( EEPROM_SPICON2bits.SEN );      // wait until start condition is over
    if ( PIR2bits.BCLIF )           // test for bus collision
        return XEE_BUS_COLLISION;                // return with Bus Collision error

    if ( WriteI2C( EEPROM_CONTROL | WRITE ) )    // write 1 byte
        return XEE_BUS_COLLISION;              // set error for write collision

    IdleI2C();                    // ensure module is idle
   if ( !EEPROM_SPICON2bits.ACKSTAT )   // test for ACK condition, if received
    {
        if ( WriteI2C( tempAddress.bytes.MSB ) )  // WRITE word address to EEPROM
            return XEE_BUS_COLLISION;            // return with write collision error

        IdleI2C();                  // ensure module is idle

        if ( WriteI2C( tempAddress.bytes.LSB ) )  // WRITE word address to EEPROM
            return XEE_BUS_COLLISION;            // return with write collision error

        IdleI2C();                  // ensure module is idle
        if ( !EEPROM_SPICON2bits.ACKSTAT ) // test for ACK condition received
            return XEE_SUCCESS;
        else
            return XEE_NAK;            // return with Not Ack error
    }
  else
        return XEE_NAK;              // return with Not Ack error
}

/*********************************************************************
 * Function:        XEE_RESULT XEEBeginRead(unsigned char control,
 *                                          XEE_ADDR address)
 *
 * PreCondition:    XEEInit() is already called.
 *
 * Input:           control - EEPROM control and address code.
 *                  address - Address at which read is to be performed.
 *
 * Output:          XEE_SUCCESS if successful
 *                  other value if failed.
 *
 * Side Effects:    None
 *
 * Overview:        Sets internal address counter to given address.
 *                  Puts EEPROM in sequential read mode.
 *
 * Note:            This function does not release I2C bus.
 *                  User must call XEEEndRead() when read is not longer
 *                  needed; I2C bus will released after XEEEndRead()
 *                  is called.
 ********************************************************************/
XEE_RESULT XEEBeginRead(XEE_ADDR address )
{
    unsigned char r;

    r = XEESetAddr(address);
    if ( r != XEE_SUCCESS )
        return r;

    r = XEEClose();
    if ( r != XEE_SUCCESS )
        return r;


    IdleI2C();
    StartI2C();
    while( EEPROM_SPICON2bits.SEN );
    if ( PIR2bits.BCLIF )
        return XEE_BUS_COLLISION;

    if ( WriteI2C(EEPROM_CONTROL | READ) )
        return XEE_BUS_COLLISION;

    IdleI2C();
    if ( !EEPROM_SPICON2bits.ACKSTAT )
        return XEE_SUCCESS;
    return XEE_NAK;
}


XEE_RESULT XEEWrite(unsigned char val)
{
    IdleI2C();                  // ensure module is idle

    if ( WriteI2C( val ) )   // data byte for EEPROM
        return XEE_BUS_COLLISION;          // set error for write collision

    IdleI2C();
    if ( !EEPROM_SPICON2bits.ACKSTAT )
        return XEE_SUCCESS;
    return XEE_NAK;
}

/*********************************************************************
 * Function:        XEE_RESULT XEEEndWrite(void)
 *
 * PreCondition:    XEEInit() && XEEBeginWrite() are already called.
 *
 * Input:           None
 *
 * Output:          XEE_SUCCESS if successful
 *                  other value if failed.
 *
 * Side Effects:    None
 *
 * Overview:        Instructs EEPROM to begin write cycle.
 *
 * Note:            Call this function after either page full of bytes
 *                  written or no more bytes are left to load.
 *                  This function initiates the write cycle.
 *                  User must call for XEEWait() to ensure that write
 *                  cycle is finished before calling any other
 *                  routine.
 ********************************************************************/
XEE_RESULT XEEEndWrite(void)
{
    IdleI2C();
    StopI2C();
    while(EEPROM_SPICON2bits.PEN);
    return XEE_SUCCESS;
}



/*********************************************************************
 * Function:        XEE_RESULT XEERead(void)
 *
 * PreCondition:    XEEInit() && XEEBeginRead() are already called.
 *
 * Input:           None
 *
 * Output:          XEE_SUCCESS if successful
 *                  other value if failed.
 *
 * Side Effects:    None
 *
 * Overview:        Reads next byte from EEPROM; internal address
 *                  is incremented by one.
 *
 * Note:            This function does not release I2C bus.
 *                  User must call XEEEndRead() when read is not longer
 *                  needed; I2C bus will released after XEEEndRead()
 *                  is called.
 ********************************************************************/
unsigned char XEERead(void)
{
    getcI2C();
    while( EEPROM_SPICON2bits.RCEN );  // check that receive sequence is over.

    EEPROM_SPICON2bits.ACKDT = 0;      // Set ack bit
    EEPROM_SPICON2bits.ACKEN = 1;
    while( EEPROM_SPICON2bits.ACKEN );

    return EEPROM_SSPBUF;
}

/*********************************************************************
 * Function:        XEE_RESULT XEEEndRead(void)
 *
 * PreCondition:    XEEInit() && XEEBeginRead() are already called.
 *
 * Input:           None
 *
 * Output:          XEE_SUCCESS if successful
 *                  other value if failed.
 *
 * Side Effects:    None
 *
 * Overview:        Ends sequential read cycle.
 *
 * Note:            This function ends seuential cycle that was in
 *                  progress.  It releases I2C bus.
 ********************************************************************/
XEE_RESULT XEEEndRead(void)
{
    getcI2C();
    while( EEPROM_SPICON2bits.RCEN );  // check that receive sequence is over.

    NotAckI2C();
    while( EEPROM_SPICON2bits.ACKEN );

    StopI2C();
    while( EEPROM_SPICON2bits.PEN );

    return XEE_SUCCESS;
}


/*********************************************************************
 * Function:        XEE_RESULT XEEReadArray(unsigned char control,
 *                                          XEE_ADDR address,
 *                                          unsigned char *buffer,
 *                                          unsigned char length)
 *
 * PreCondition:    XEEInit() is already called.
 *
 * Input:           control     - EEPROM control and address code.
 *                  address     - Address from where array is to be read
 *                  buffer      - Caller supplied buffer to hold the data
 *                  length      - Number of bytes to read.
 *
 * Output:          XEE_SUCCESS if successful
 *                  other value if failed.
 *
 * Side Effects:    None
 *
 * Overview:        Reads desired number of bytes in sequential mode.
 *                  This function performs all necessary steps
 *                  and releases the bus when finished.
 *
 * Note:            None
 ********************************************************************/
XEE_RESULT XEEReadArray(XEE_ADDR address,
                        unsigned char *buffer,
                        unsigned char length)
{
    XEE_RESULT r;

    r = XEEBeginRead(address);
    if ( r != XEE_SUCCESS )
        return r;

    while( length-- )
        *buffer++ = XEERead();

    r = XEEEndRead();

    return r;
}




XEE_RESULT XEEClose(void)
{
    IdleI2C();
    StopI2C();
    while( EEPROM_SPICON2bits.PEN );           // wait until stop condition is over.
    if ( PIR2bits.BCLIF )
        return XEE_BUS_COLLISION;
    return XEE_SUCCESS;
}



/*********************************************************************
 * Function:        XEE_RESULT XEEIsBusy(unsigned char control)
 *
 * PreCondition:    XEEInit() is already called.
 *
 * Input:           control     - EEPROM control and address code.
 *
 * Output:          XEE_READY if EEPROM is not busy
 *                  XEE_BUSY if EEPROM is busy
 *                  other value if failed.
 *
 * Side Effects:    None
 *
 * Overview:        Requests ack from EEPROM.
 *
 * Note:            None
 ********************************************************************/
XEE_RESULT XEEIsBusy(void)
{
    XEE_RESULT r;

    IdleI2C();                      // ensure module is idle
    StartI2C();                     // initiate START condition
    while ( EEPROM_SPICON2bits.SEN );      // wait until start condition is over
    if ( PIR2bits.BCLIF )           // test for bus collision
    {
        return XEE_BUS_COLLISION;                // return with Bus Collision error
    }

    else
    {
        if ( WriteI2C( EEPROM_CONTROL | WRITE ) )    // write byte - R/W bit should be 0
            return XEE_BUS_COLLISION;              // set error for write collision

        IdleI2C();                    // ensure module is idle
        if ( PIR2bits.BCLIF )         // test for bus collision
            return XEE_BUS_COLLISION;              // return with Bus Collision error

        if ( !EEPROM_SPICON2bits.ACKSTAT )
            r = XEE_READY;
        else
            r = XEE_BUSY;

#if 0
        while ( EEPROM_SPICON2bits.ACKSTAT ) // test for ACK condition received
        {
            RestartI2C();               // initiate Restart condition
            while ( EEPROM_SPICON2bits.RSEN ); // wait until re-start condition is over
            if ( PIR2bits.BCLIF )       // test for bus collision
                return XEE_BUS_COLLISION;            // return with Bus Collision error

            if ( WriteI2C( control ) )  // write byte - R/W bit should be 0
                return XEE_BUS_COLLISION;            // set error for write collision

            IdleI2C();                  // ensure module is idle
        }
#endif
    }


    StopI2C();                      // send STOP condition
    while ( EEPROM_SPICON2bits.PEN );      // wait until stop condition is over
    if ( PIR2bits.BCLIF )           // test for bus collision
        return XEE_BUS_COLLISION;                // return with Bus Collision error

    return r;
    //return XEE_READY;                   // return with no error
}


/********************************************************************
*     Function Name:    WriteI2C                                    *
*     Return Value:     Status byte for WCOL detection.             *
*     Parameters:       Single data byte for I2C bus.               *
*     Description:      This routine writes a single byte to the    *
*                       I2C bus.                                    *
********************************************************************/
static unsigned char WriteI2C( unsigned char data_out )
{
  EEPROM_SSPBUF = data_out;           // write single byte to EEPROM_SSPBUF
  if ( SSPCON1bits.WCOL )      // test if write collision occurred
   return ( -1 );              // if WCOL bit is set return negative #
  else
  {
    while( SSPSTATbits.BF );   // wait until write cycle is complete
    return ( 0 );              // if WCOL bit is not set return non-negative #
  }
}



/********************************************************************
*     Function Name:    ReadI2C                                     *
*     Return Value:     contents of EEPROM_SSPBUF register                 *
*     Parameters:       void                                        *
*     Description:      Read single byte from I2C bus.              *
********************************************************************/
static unsigned char getcI2C( void )
{
	EEPROM_SPICON2bits.RCEN = 1;		// enable master for 1 byte reception
	while(!SSPSTATbits.BF);		// wait until byte received
	return EEPROM_SSPBUF;				// return with read byte
}


#endif //#if defined(MPFS_USE_EEPROM)

#endif  //#if defined(NEW_I2C_Routines)
